#!/usr/bin/perl -w
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.


use strict;
use Tie::IxHash;
package lib::Kerberos;


# subroutine to create/update .cache_k5login file and create a mapping in krb5.conf
# this enables configuration when Kerberos principal does not match login userid
sub set_remoteuserid {

    my ($username, $remoteuserid) = @_;
    my $krb5conf = "/etc/krb5.conf";
    my $contents = "";
    my $oldmappings = "";
    my $oldpam = "";
    my %mappings_hash = ();    
    tie %mappings_hash, "Tie::IxHash";
    my $newmappings = "";
    my $newpam = "";
    my $key = 0;
    my $value = "";
    my $root_uid = 0;
    my $quoted = "";
    
    # always cache remoteuserid to .cache_k5login (regardless of whether Kerberos is enabled or not)
    # this will allow mappings to be created in krb5.conf if Kerberos is configured AFTER Kerberos user is created
    my $echo = `echo $remoteuserid > /home/$username/.cache_k5login`;
    chmod 0644, '/home/$username/.cache_k5login';
    chown $root_uid, $root_uid, '/home/$username/.cache_k5login';
    
    # if krb5.conf exists, add/update mappings there
    if (-e $krb5conf) {
	unless (open (FILEHANDLE, "+< $krb5conf")){
	    return;
	}    
        flock FILEHANDLE, 2;
        undef $/;
        $contents = <FILEHANDLE>;

	# create "mappings" tag if it does not exist
	# pam section must exist unless this is initial install and Kerberos has not been configured yet, then we don't do anything
	if ($contents !~ m/mappings/) {
	    if ($contents =~ m/(\n)(( |\t)*)pam = (.|\s)*?\}/) {
		$oldpam = "$&";
		$newpam = "$&";
		$newpam =~ s/\}/\tmappings = \n\t\}/;
		$contents =~ s/$oldpam/$newpam/;
	    }
	}

	# fill in mappings
	if ($contents =~ m/mappings = (.*)(\n)/) {
            $oldmappings = "$&";
	    chop $oldmappings;
	    # convert string into hash so that there is no confusion about keys and values
	    %mappings_hash = split(/ /,$oldmappings);
	    # update the value of hash element or create new hash element
	    # add ^ to the beginning and $ to the end of the username to match exact string only (defect 158470)
	    $username = '^'.$username.'$';
	    $mappings_hash{$username} = $remoteuserid;
	    # convert hash back into string
	    foreach $key (keys %mappings_hash) {
		$value = $mappings_hash{$key};
		$newmappings .= "$key $value ";
	    }
	    # replace old mappings with new
	    # add backslashes for substitute to work
	    $quoted = quotemeta ($oldmappings);
	    $contents =~ s/$quoted/$newmappings\n/;
	}
	
	#remove extra new lines
	$contents =~ s/\n\n/\n/g;
	#write changed contents back into file
	seek FILEHANDLE, 0, 0;
	truncate FILEHANDLE, 0;
	print FILEHANDLE $contents;
	close FILEHANDLE;	
    }
}


# subroutine to delete .cache_k5login file and remove user's mapping from krb5.conf file
# used by rmuser and chuser (a change from kerberos to local or ldap usertype)
sub remove_remoteuserid {

    my ($username) = @_;
    my $krb5conf = "/etc/krb5.conf";
    my $contents = "";
    my $oldmappings = "";
    my %mappings_hash = ();    
    tie %mappings_hash, "Tie::IxHash";
    my $newmappings = "";
    my $key = 0;
    my $value = "";
    my $quoted ="";
      
    # delete .cache_k5login file
    if ( -e "/home/$username/.cache_k5login") {
	unlink "/home/$username/.cache_k5login";
    }
           
    # if krb5.conf exists, delete this user's mapping
    if (-e $krb5conf) {
	unless (open (FILEHANDLE, "+< $krb5conf")){
	    return;
	}    
        flock FILEHANDLE, 2;
        undef $/;
        $contents = <FILEHANDLE>;

	# "mappings" tag should already be in krb5.conf file unless Kerberos is not configured (initial shipped configuration only);
	# in which case we don't do anything
	if ($contents =~ m/mappings = (.*)(\n)/) {
            $oldmappings = "$&";
	    chop $oldmappings;
	    # convert string into hash so that there is no confusion about keys and values
	    %mappings_hash = split(/ /,$oldmappings);
	    # delete the mapping if it exists
	    # username is in the format ^username$ to ensure unique mapping (defect 158470)
	    $username = '^'.$username.'$';
            if (exists ($mappings_hash{$username})) {
                delete ($mappings_hash{$username});    
            }
	    # convert hash back into string
	    foreach $key (keys %mappings_hash) {
		$value = $mappings_hash{$key};
		$newmappings .= "$key $value ";
	    }
	    # replace old mappings with new
	    $quoted = quotemeta ($oldmappings);
	    $contents =~ s/$quoted/$newmappings\n/;
	    
	    # if this was the last mapping, delete "mappings" tag
	    if ($contents =~ m/(( |\t)*)mappings = (( |\t)*)(\n)/) {
		$contents =~ s/(( |\t)*)mappings =(.*)\n//;
	    }
	}
	
	#remove extra new lines
	$contents =~ s/\n\n/\n/g;
	#write changed contents back into file
	seek FILEHANDLE, 0, 0;
	truncate FILEHANDLE, 0;
	print FILEHANDLE $contents;
	close FILEHANDLE;	
    }   
}

1;
